#ifndef __SW_64_PCI_H
#define __SW_64_PCI_H

#ifdef __KERNEL__

#include <linux/spinlock.h>
#include <linux/dma-mapping.h>
#include <linux/scatterlist.h>
#include <asm/machvec.h>

/*
 * The following structure is used to manage multiple PCI busses.
 */

struct pci_dev;
struct pci_bus;
struct resource;
struct pci_iommu_arena;
struct sw_iommu;
struct page;


/* A controller.  Used to manage multiple PCI busses.  */

struct pci_controller {
	struct pci_controller *next;
	struct pci_bus *bus;
	struct resource *io_space;
	struct resource *mem_space;
	struct resource *pre_mem_space;
	unsigned long sparse_mem_base;
	unsigned long dense_mem_base;
	unsigned long sparse_io_base;
	unsigned long dense_io_base;

	/* This one's for the kernel only.  It's in KSEG somewhere.  */
	unsigned long config_space_base;

	unsigned int index;
	/* For compatibility with current (as of July 2003) pciutils
	   and XFree86. Eventually will be removed. */
	unsigned int need_domain_info;

	struct pci_iommu_arena *sg_pci;
	struct pci_iommu_arena *sg_isa;

#ifdef CONFIG_IOMMU
	bool iommu_enable; /* For SW4A IOMMU */
#endif

	void *sysdata;
};

/* Override the logic in pci_scan_bus for skipping already-configured
   bus numbers.  */

#define pcibios_assign_all_busses()	1

#define PCIBIOS_MIN_IO		sw_mv.min_io_address
#define PCIBIOS_MIN_MEM		sw_mv.min_mem_address

extern void pcibios_set_master(struct pci_dev *dev);

/* IOMMU controls.  */


/* TODO: integrate with include/asm-generic/pci.h ? */
static inline int pci_get_legacy_ide_irq(struct pci_dev *dev, int channel)
{
	return channel ? 15 : 14;
}

#define pci_domain_nr(bus) ((struct pci_controller *)(bus)->sysdata)->index

static inline int pci_proc_domain(struct pci_bus *bus)
{
	struct pci_controller *hose = bus->sysdata;

	return hose->need_domain_info;
}

#endif /* __KERNEL__ */

/* Values for the `which' argument to sys_pciconfig_iobase.  */
#define IOBASE_HOSE		0
#define IOBASE_SPARSE_MEM	1
#define IOBASE_DENSE_MEM	2
#define IOBASE_SPARSE_IO	3
#define IOBASE_DENSE_IO		4
#define IOBASE_ROOT_BUS		5
#define IOBASE_FROM_HOSE	0x10000

extern struct pci_dev *isa_bridge;

extern int pci_legacy_read(struct pci_bus *bus, loff_t port, u32 *val,
			   size_t count);
extern int pci_legacy_write(struct pci_bus *bus, loff_t port, u32 val,
			    size_t count);
extern int pci_mmap_legacy_page_range(struct pci_bus *bus,
				      struct vm_area_struct *vma,
				      enum pci_mmap_state mmap_state);
extern void pci_adjust_legacy_attr(struct pci_bus *bus,
				   enum pci_mmap_state mmap_type);
#define HAVE_PCI_LEGACY	1

extern int pci_create_resource_files(struct pci_dev *dev);
extern void pci_remove_resource_files(struct pci_dev *dev);

extern void reserve_mem_for_pci(void);
/* To support multiple MSI override arch_setup_msi_irqs() */
#define arch_setup_msi_irqs sw_setup_msi_irqs

#define PCI_VENDOR_ID_JN          0x5656
#define PCI_DEVICE_ID_JN_PCIESW   0x1000
#define PCI_DEVICE_ID_JN_PCIEUSIP 0x1200
#define PCI_DEVICE_ID_JN_PCIE2PCI 0x1314

#if 0
#define PCI_DEVICE_ID_JN_USBEHCI  0x0052
#define PCI_DEVICE_ID_JN_USBOHCI0 0x0050
#define PCI_DEVICE_ID_JN_USBOHCI1 0x0051
#define PCI_DEVICE_ID_JN_AC97     0x0060
#define PCI_DEVICE_ID_JN_VGA      0x0100
#define PCI_DEVICE_ID_JN_SATA     0x0110
#define PCI_DEVICE_ID_JN_GMAC0    0x0120
#define PCI_DEVICE_ID_JN_GMAC1    0x0121
#define PCI_DEVICE_ID_JN_TCM      0x0160
#define PCI_DEVICE_ID_JN_I2C      0x0240
#define PCI_DEVICE_ID_JN_SPI      0x0250
#define PCI_DEVICE_ID_JN_XBUS     0x0260
#define PCI_DEVICE_ID_JN_PMDEBUG  0x0270
#define PCI_DEVICE_ID_JN_BT       0x0280
#define PCI_DEVICE_ID_JN_KCS      0x0290
#define PCI_DEVICE_ID_JN_UART0    0x0300
#define PCI_DEVICE_ID_JN_UART1    0x0301
#define PCI_DEVICE_ID_JN_PS20     0x0310
#define PCI_DEVICE_ID_JN_PS21     0x0311
#endif

/* Device IDs for SICH  */
#define PCI_DEVICE_ID_SICH_XHCI         0x0009
#define PCI_DEVICE_ID_SICH_LPC          0x0017
#define PCI_DEVICE_ID_SICH_GPIO         0x0015
#define PCI_DEVICE_ID_SICH_KBD          0x001F
#define PCI_DEVICE_ID_SICH_MUX          0x011F
#define PCI_DEVICE_ID_SICH_UART         0x001E
#define PCI_DEVICE_ID_SICH_SPI          0x0019
#define PCI_DEVICE_ID_SICH_I2C0         0x0018
#define PCI_DEVICE_ID_SICH_I2C1         0x0118
#define PCI_DEVICE_ID_SICH_I2C2         0x0218
#define PCI_DEVICE_ID_SICH_GMAC0        0x000C
#define PCI_DEVICE_ID_SICH_GMAC1        0x010C
#define PCI_DEVICE_ID_SICH_SATA         0x000B
#define PCI_DEVICE_ID_SICH_VIDEO        0x000A
#define PCI_DEVICE_ID_SICH_AUDIO        0x0006

#define NR_IRQ_VECTORS NR_IRQS

#define LAST_DEVICE_VECTOR  31

#define PCITODMA_OFFSET	0x0    /*0 offset*/

#endif /* __SW_64_PCI_H */
