/*
 * =====================================================================================
 *       Filename:  sw.h
 *    Description:
 *        Created:  2009年02月28日 11时23分49秒
 * =====================================================================================
 */

#ifndef _SW_H_
#define  _SW_H_

#include <asm/compiler.h>
#include <asm/cache.h>		/* we use L1_CACHE_BYTES */
#include <asm/hwrpb.h>
#include <asm/errno.h>

#define SOFTINF_IOADDR(cpuid) ((0x1UL<<39) | (0x5UL<<36) | ((0x13UL + (unsigned long)cpuid) << 7) | 0xfffffc0000000000UL)

#define PAL_sw__swpipl	53
#define PAL_sendii       0x3E

#define __SYS_CALL_RW1(NAME, RTYPE, TYPE0)			\
static inline RTYPE NAME(TYPE0 arg0)				\
{								\
	register RTYPE __r0 __asm__("$0");			\
	register TYPE0 __r16 __asm__("$16") = arg0;		\
	__asm__ __volatile__(					\
		"sys_call %2 # "#NAME				\
		: "=r"(__r16), "=r"(__r0)			\
		: "i"(PAL_ ## NAME), "0"(__r16)			\
		: "$1", "$22", "$23", "$24", "$25");		\
	return __r0;						\
}
#define __CALL_PAL_RW3(NAME, RTYPE, TYPE0, TYPE1, TYPE2)                \
	static inline RTYPE NAME(TYPE0 arg0, TYPE1 arg1, TYPE2 arg2)            \
{                                                               \
	register RTYPE __r0 __asm__("$0");                      \
	register TYPE0 __r16 __asm__("$16") = arg0;             \
	register TYPE1 __r17 __asm__("$17") = arg1;             \
	register TYPE2 __r18 __asm__("$18") = arg2;             \
	__asm__ __volatile__(                                   \
			"sys_call %4 # "#NAME                           \
			: "=r"(__r16), "=r"(__r17), "=r"(__r18), "=r"(__r0)              \
			: "i"(PAL_ ## NAME), "0"(__r16), "1"(__r17), "2"(__r18) \
			: "$1", "$22", "$23", "$24", "$25");            \
	return __r0;                                            \
}


__SYS_CALL_RW1(sw__swpipl, unsigned long, unsigned long);

#define sw__setipl(ipl)		((void) sw__swpipl(ipl))

#define sw_irq_save(flags)	do { (flags) = sw__swpipl(7); barrier(); } while (0)
#define sw_irq_restore(flags)	do { barrier(); sw__setipl(flags); barrier(); } while (0)


typedef struct {
	volatile unsigned int lock;
} sw_spinlock_t;
#define __SW_SPIN_LOCK_UNLOCKED        { 0 }

extern sw_spinlock_t sw_bit_lock;      /* set_bit函数的锁 */

static inline unsigned int sw_swpipl(unsigned int ipl)
{
	register  unsigned long __v0 __asm__("$0");
	__asm__ __volatile__(
		"sys_call  0x35\n"
	);

	return __v0;
}

static inline void sw_spin_unlock(sw_spinlock_t *lock)
{
	__asm__("memb");
	lock->lock = 0;
}

static inline unsigned long sw_getcpuid(void)
{
	register  unsigned long __v0 __asm__("$0");
	__asm__ __volatile__(
		"sys_call  0x90\n"
 : "=r"(__v0)
		:
	);

	return __v0;
}

static inline void sw_spin_lock(sw_spinlock_t *lock)
{
	long tmp;
       __asm__ __volatile__(
	"	memb\n"
	"1:     ldi %0,%1\n"
	"       ldw_set %0,0(%0)\n"	/* 取出锁地址内容，并对锁地址置1 */
	"       bne     %0,1b\n"        /* 如果不返回0（为1），则表示锁已被占住 */
	"       memb\n"                 /* 如果返回0，则占用锁，并上锁. */
	: "=&r" (tmp), "=m" (lock->lock)
	: "m"(lock->lock) : "memory");
}
/* Use an array of spinlocks for our atomic_ts.
 * Hash function to index into a different SPINLOCK.
 * Since "a" is usually an address, use one spinlock per cacheline.
 */

static inline void ldw_inc(unsigned int *addr)
{
	long tmp;

	__asm__ __volatile__(
	"       memb\n"
	"1:	ldi %0,%1\n"
	"	ldw_inc	%0,0(%0)\n"        /* 取出锁地址内容，并对锁地址置1 */
	"       memb\n"		/* 如果返回0，则占用锁，并上锁. */
	: "=&r" (tmp), "=m" (*addr)
	: "m"(*addr) : "memory");
}

static inline void ldl_inc(unsigned long *addr)
{
	long tmp;

	__asm__ __volatile__(
	"       memb\n"
	"1:	ldi %0,%1\n"
	"	ldl_inc	%0,0(%0)\n"        /* 取出锁地址内容，并对锁地址置1 */
	"       memb\n"		/* 如果返回0，则占用锁，并上锁. */
	: "=&r" (tmp), "=m" (*addr)
	: "m"(*addr) : "memory");
}


static inline void ldl_dec(unsigned long *addr)
{
	long tmp;

	__asm__ __volatile__(
	"       memb\n"
	"1:	ldi %0,%1\n"
	"	ldl_dec	%0,0(%0)\n"        /* 取出锁地址内容，并对锁地址置1 */
	"       memb\n"		/* 如果返回0，则占用锁，并上锁. */
	: "=&r" (tmp), "=m" (*addr)
	: "m"(*addr) : "memory");
}


static inline void ldw_dec(unsigned int *addr)
{
	long tmp;

	__asm__ __volatile__(
	"       memb\n"
	"1:	ldi %0,%1\n"
	"	ldw_dec	%0,0(%0)\n"        /* 取出锁地址内容，并对锁地址置1 */
	"       memb\n"		/* 如果返回0，则占用锁，并上锁. */
	: "=&r" (tmp), "=m" (*addr)
	: "m"(*addr) : "memory");
}


static inline void ldw_set(unsigned int *addr)
{
	long tmp;

	__asm__ __volatile__(
	"       memb\n"
	"1:	ldi %0,%1\n"
	"	ldw_set	%0,0(%0)\n"        /* 取出锁地址内容，并对锁地址置1 */
	"       memb\n"		/* 如果返回0，则占用锁，并上锁. */
	: "=&r" (tmp), "=m" (*addr)
	: "m"(*addr) : "memory");
}


static inline void ldl_set(unsigned long *addr)
{
	long tmp;

	__asm__ __volatile__(
	"       memb\n"
	"1:	ldi %0,%1\n"
	"	ldl_set	%0,0(%0)\n"        /* 取出锁地址内容，并对锁地址置1 */
	"       memb\n"		/* 如果返回0，则占用锁，并上锁. */
	: "=&r" (tmp), "=m" (*addr)
	: "m"(*addr) : "memory");
}


extern void __send_ipi_message(int pcpuid, void *message);
#endif   /* ----- #ifndef _SW_H_  ----- */

