#ifndef _SW6_IO_H_
#define  _SW6_IO_H_

#include <asm/page.h>
#include <asm/io.h>

#ifdef CONFIG_SW_4LEVEL_PGTABLE
#define IO_MARK_BIT  47
#else
#define IO_MARK_BIT  39
#endif

#define IO_BITS         (PAGE_OFFSET | (0x1UL<<IO_MARK_BIT))
#define MIU_IO_BASE     (0x5UL<<36)
#define IPU_IO_BASE     (0x6UL<<36)

enum MIU_REG {
       CG_ONLINE           = IO_BITS | MIU_IO_BASE |   (0x00000000)<<7,    /* 核组在位使能寄存器 */
       CG0_FAULT_STAT      = IO_BITS | MIU_IO_BASE |   (0x00000001)<<7,    /* CG0核组总错状态寄存器 */
       CG1_FAULT_STAT      = IO_BITS | MIU_IO_BASE |   (0x00000002)<<7,    /* CG1核组总错状态寄存器 */
       CG2_FAULT_STAT      = IO_BITS | MIU_IO_BASE |   (0x00000003)<<7,    /* CG2核组总错状态寄存器 */
       CG3_FAULT_STAT      = IO_BITS | MIU_IO_BASE |   (0x00000004)<<7,    /* CG3核组总错状态寄存器 */
       CG0_FAULT_EN        = IO_BITS | MIU_IO_BASE |   (0x00000005)<<7,    /* CG0核组总错状态报错使能寄存器 */
       CG1_FAULT_EN        = IO_BITS | MIU_IO_BASE |   (0x00000006)<<7,    /* CG1核组总错状态报错使能寄存器 */
       CG2_FAULT_EN        = IO_BITS | MIU_IO_BASE |   (0x00000007)<<7,    /* CG2核组总错状态报错使能寄存器 */
       CG3_FAULT_EN        = IO_BITS | MIU_IO_BASE |   (0x00000008)<<7,    /* CG3核组总错状态报错使能寄存器 */
       CG0_FAULT_INT_EN    = IO_BITS | MIU_IO_BASE |   (0x00000009)<<7,    /* CG0核组故障中断使能寄存器 */
       CG1_FAULT_INT_EN    = IO_BITS | MIU_IO_BASE |   (0x0000000a)<<7,    /* CG1核组故障中断使能寄存器 */
       CG2_FAULT_INT_EN    = IO_BITS | MIU_IO_BASE |   (0x0000000b)<<7,    /* CG2核组故障中断使能寄存器 */
       CG3_FAULT_INT_EN    = IO_BITS | MIU_IO_BASE |   (0x0000000c)<<7,    /* CG3核组故障中断使能寄存器 */
       INIT_CTL            = IO_BITS | MIU_IO_BASE |   (0x0000000d)<<7,    /* 初始化控制寄存器 */
       CORE_ONLINE         = IO_BITS | MIU_IO_BASE |   (0x0000000f)<<7,    /* 核心在位使能寄存器 */
       CPUID               = IO_BITS | MIU_IO_BASE |   (0x00000012)<<7,    /* CPU 号寄存器 */
       LONG_TIME           = IO_BITS | MIU_IO_BASE |   (0x00000023)<<7,    /* 长时钟计数寄存器 */
       LONG_PRESET         = IO_BITS | MIU_IO_BASE |   (0x00000024)<<7,    /* 长时钟计数间隔寄存器 */
       SI_FAULT_STAT       = IO_BITS | MIU_IO_BASE |   (0x00000062)<<7,    /* 系统接口总错状态寄存器 */
       SI_FAULT_EN         = IO_BITS | MIU_IO_BASE |   (0x00000063)<<7,    /* 系统接口总错使能寄存器 */
       SI_FAULT_INT_EN     = IO_BITS | MIU_IO_BASE |   (0x00000064)<<7,    /* 系统接口故障中断使能寄存器 */
       MC_ONLINE           = IO_BITS | MIU_IO_BASE |   (0x0000006f)<<7,    /* 存控在位使能寄存器 */
       MC_CONFIG           = IO_BITS | MIU_IO_BASE |   (0x00000092)<<7     /* 存控配置寄存器 */
};

enum IPU_REG {
       SHORT_CLK_START     = IO_BITS | IPU_IO_BASE |   (0x000000004)<<7,    /* 短时钟中断计数起始值寄存器 */
       SHORT_CLK_MSK       = IO_BITS | IPU_IO_BASE |   (0x000000005)<<7,    /* 短时钟中断屏蔽寄存器 */
       INTEN               = IO_BITS | IPU_IO_BASE |   (0x000000008)<<7,    /* 中断使能寄存器 */
       SYSERR_NMI_INT_DST  = IO_BITS | IPU_IO_BASE |   (0x000000017)<<7	    /* 系统故障和不可屏蔽中断的目标配置寄存器 */
};

#define SWIO_READ_PCIE0_MSI_INDX(reg) readq((void *) __iomem(IO_BITS | IPU_IO_BASE |   (0x00000100UL + reg)<<7))
#define SWIO_WRITE_PCIE0_MSI_INDX(reg, val)  writeq(val, (void *) __iomem (IO_BITS | IPU_IO_BASE |   (0x00000100UL + reg)<<7))

#define SWIO_READ_PCIE1_MSI_INDX(reg) readq((void *) __iomem(IO_BITS | IPU_IO_BASE |   (0x00000200UL + reg)<<7))
#define SWIO_WRITE_PCIE1_MSI_INDX(reg, val)  writeq(val, (void *) __iomem (IO_BITS | IPU_IO_BASE |   (0x00000200UL + reg)<<7))

#define SWIO_READ_REG(reg) readq((void *) __iomem reg)
#define SWIO_WRITE_REG(reg, val)  writeq(val, (void *) __iomem reg)
#endif
